@@ -1008,6 +1008,10 @@ gnome_vfs_mime_type_is_known (const char *mime_type)
 
 	reload_if_needed ();
 
+	if (is_mime_type_deleted (mime_type)) {
+		return FALSE;
+	}
+	
 	if (g_hash_table_lookup (specific_types, mime_type)) {
 		return TRUE;
 	}
@@ -1142,6 +1146,10 @@ GnomeVFSResult
 gnome_vfs_mime_set_extensions_list (const char *mime_type,
 				    const char *extensions_list)
 {
+	if (is_mime_type_deleted (mime_type)) {
+		gnome_vfs_mime_set_registered_type_key (mime_type, DELETED_KEY, "");
+	}
+
 	return gnome_vfs_mime_set_registered_type_key (mime_type, "ext", extensions_list);
 }
 
