@@ -97,7 +97,7 @@ GET_PATH_MAX (void)
 #define OPEN open
 #endif
 
-#ifdef HAVE_LSEEK64
+#if defined(HAVE_LSEEK64) && defined(HAVE_OFF64_T)
 #define LSEEK lseek64
 #define OFF_T off64_t
 #else
@@ -582,6 +582,7 @@ read_link (const gchar *full_name)
 
                 read_size = readlink (full_name, buffer, size);
 		if (read_size < 0) {
+			g_free (buffer);
 			return NULL;
 		}
                 if (read_size < size) {
@@ -627,6 +628,8 @@ get_stat_info (GnomeVFSFileInfo *file_info,
 	gboolean recursive;
 	char *link_file_path;
 	char *symlink_name;
+	char *symlink_dir;
+	char *newpath;
 	
 	followed_symlink = FALSE;
 	
@@ -684,6 +687,14 @@ get_stat_info (GnomeVFSFileInfo *file_info,
 				g_free (link_file_path);
 				return gnome_vfs_result_from_errno ();
 			}
+			if (symlink_name[0] != '/') {
+				symlink_dir = g_path_get_dirname (link_file_path);
+				newpath = g_build_filename (symlink_dir,
+							    symlink_name, NULL);
+				g_free (symlink_dir);
+				g_free (symlink_name);
+				symlink_name = newpath;
+			}
 			
 			if ((options & GNOME_VFS_FILE_INFO_FOLLOW_LINKS) == 0
 			                /* if we had an earlier ELOOP, don't get in an infinite loop here */
@@ -771,6 +782,10 @@ do_close_directory (GnomeVFSMethod *method,
 	return GNOME_VFS_OK;
 }
 
+#ifndef HAVE_READDIR_R
+G_LOCK_DEFINE_STATIC (readdir);
+#endif
+
 static GnomeVFSResult
 do_read_directory (GnomeVFSMethod *method,
 		   GnomeVFSMethodHandle *method_handle,
@@ -785,6 +800,7 @@ do_read_directory (GnomeVFSMethod *method,
 	handle = (DirectoryHandle *) method_handle;
 	
 	errno = 0;
+#ifdef HAVE_READDIR_R	
 	if (readdir_r (handle->dir, handle->current_entry, &result) != 0) {
 		/* Work around a Solaris bug.
 		 * readdir64_r returns -1 instead of 0 at EOF.
@@ -794,6 +810,21 @@ do_read_directory (GnomeVFSMethod *method,
 		}
 		return gnome_vfs_result_from_errno ();
 	}
+#else
+	G_LOCK (readdir);
+	errno = 0;
+	result = readdir (handle->dir);
+
+	if (result == NULL && errno != 0) {
+		GnomeVFSResult ret = gnome_vfs_result_from_errno ();
+		G_UNLOCK (readdir);
+		return ret;
+	}
+	if (result != NULL) {
+		memcpy (handle->current_entry, result, sizeof (struct dirent));
+	}
+	G_UNLOCK (readdir);
+#endif
 	
 	if (result == NULL) {
 		return GNOME_VFS_ERROR_EOF;
@@ -926,7 +957,9 @@ do_is_local (GnomeVFSMethod *method,
 		struct stat statbuf;
 		if (stat (path, &statbuf) == 0) {
 			char *type = filesystem_type (path, path, &statbuf);
-			gboolean is_local = strcmp (type, "nfs") && strcmp (type, "afs");
+			gboolean is_local = ((strcmp (type, "nfs") != 0) && 
+					     (strcmp (type, "afs") != 0) &&
+					     (strcmp (type, "ncpfs") != 0));
 			local = GINT_TO_POINTER (is_local ? 1 : -1);
 			g_hash_table_insert (fstype_hash, path, local);
 		}
@@ -1094,18 +1127,38 @@ find_trash_in_one_hierarchy_level (const char *current_directory, dev_t near_dev
 
 	item_buffer = g_malloc (sizeof (struct dirent) + GET_PATH_MAX() + 1);
 	for (;;) {
+#ifdef HAVE_READDIR_R
 		if (readdir_r (directory, item_buffer, &item) != 0 || item == NULL) {
 			break;
 		}
-
-		if (gnome_vfs_context_check_cancellation (context))
+#else
+		G_LOCK (readdir);
+		item = readdir (directory);
+		if (item == NULL) {
+			G_UNLOCK (readdir);
 			break;
+		}
+#endif
+
+		if (gnome_vfs_context_check_cancellation (context)) {
+#ifndef HAVE_READDIR_R
+			G_UNLOCK (readdir);
+#endif
+			break;
+		}
 
 		if (strcmp (item->d_name, ".") == 0
-			|| strcmp (item->d_name, "..") == 0)
+			|| strcmp (item->d_name, "..") == 0) {
+#ifndef HAVE_READDIR_R
+			G_UNLOCK (readdir);
+#endif
 			continue;
+		}
 
 		item_path = append_to_path (current_directory, item->d_name);
+#ifndef HAVE_READDIR_R
+		G_UNLOCK (readdir);
+#endif
 		if (lstat (item_path, &stat_buffer) == 0 
 			&& S_ISDIR (stat_buffer.st_mode)
 			&& near_device_id == stat_buffer.st_dev) {
@@ -2073,7 +2126,7 @@ fam_do_iter_unlocked (void)
 		if (FAMNextEvent(fam_connection, &ev) != 1) {
 			FAMClose(fam_connection);
 			g_free(fam_connection);
-			fam_connection = FALSE;
+			fam_connection = NULL;
 			return FALSE;
 		}
 
@@ -2206,22 +2259,26 @@ do_monitor_add (GnomeVFSMethod *method,
 	gnome_vfs_uri_ref (uri);
 	filename = get_path_from_uri (uri);
 
+	G_LOCK (fam_connection);
 	/* We need to queue up incoming messages to avoid blocking on write
 	   if there are many monitors being added */
 	fam_do_iter_unlocked ();
+
+	if (fam_connection == NULL) {
+		G_UNLOCK (fam_connection);
+		return GNOME_VFS_ERROR_NOT_SUPPORTED;
+	}
 	
 	if (monitor_type == GNOME_VFS_MONITOR_FILE) {
-		G_LOCK (fam_connection);
 		FAMMonitorFile (fam_connection, filename, 
 			&handle->request, handle);
-		G_UNLOCK (fam_connection);
 	} else {
-		G_LOCK (fam_connection);
 		FAMMonitorDirectory (fam_connection, filename, 
 			&handle->request, handle);
-		G_UNLOCK (fam_connection);
 	}
 
+	G_UNLOCK (fam_connection);
+	
 	*method_handle_return = (GnomeVFSMethodHandle *)handle;
 
 	g_free (filename);
@@ -2252,6 +2309,11 @@ do_monitor_cancel (GnomeVFSMethod *method,
 	/* We need to queue up incoming messages to avoid blocking on write
 	   if there are many monitors being canceled */
 	fam_do_iter_unlocked ();
+
+	if (fam_connection == NULL) {
+		G_UNLOCK (fam_connection);
+		return GNOME_VFS_ERROR_NOT_SUPPORTED;
+	}
 	
 	FAMCancelMonitor (fam_connection, &handle->request);
 	G_UNLOCK (fam_connection);
