@@ -706,33 +706,32 @@ create_handle (GnomeVFSURI *uri,
  * Here's how the gconf gnome-vfs HTTP proxy variables
  * are intended to be used
  *
- * /system/gnome-vfs/use-http-proxy	
+ * /system/http_proxy/use_http_proxy	
  * 	Type: boolean
  *	If set to TRUE, the client should use an HTTP proxy to connect to all
- *	servers that are not "localhost".  The proxy is specified in other
- *	gconf variables below
+ *	servers. The proxy is specified in other gconf variables below.
  *
- * /system/gnome-vfs/http-proxy-host
+ * /system/http_proxy/host
  *	Type: string
- *	The hostname of the HTTP proxy this client should use.  If use-http-proxy
- *	is TRUE, this should be set.  If it is not set, the application should
- *	behave as if use-http-proxy is was set to FALSE.
+ *	The hostname of the HTTP proxy this client should use.  If
+ *	use-http-proxy is TRUE, this should be set.  If it is not set, the
+ *	application should behave as if use-http-proxy is was set to FALSE.
  *
- * /system/gnome-vfs/http-proxy-port
+ * /system/http_proxy/port
  *	Type: int
  *	The port number on the HTTP proxy host that the client should connect to
- *	If use-http-proxy and http-proxy-host are set but this is not set, 
- *	the application should use a default port value of 8080
+ *	If use_http_proxy and host are set but this is not set, the application
+ *	should use a default port value of 8080
  *
- * /system/gnome-vfs/http-proxy-authorization-user
+ * /system/http_proxy/authentication-user
  *	Type: string
  *	Username to pass to an authenticating HTTP proxy.
  *
- * /system/gnome-vfs/http-proxy-authorization-password
+ * /system/http_proxy/authentication_password
  *	Type: string
  *	Password to pass to an authenticating HTTP proxy.
  *  
- * /system/gnome-vfs/use-http-proxy-authorization
+ * /system/http_proxy/use-authentication
  *	Type: boolean
  * 	TRUE if the client should pass http-proxy-authorization-user and
  *	http-proxy-authorization-password an HTTP proxy
@@ -1967,16 +1966,24 @@ process_propfind_response(xmlNodePtr n,
 					    (0 == null_handling_strcmp (second_base->text, uri->text))) {
 						file_info->name = NULL; /* this file is the . directory */
 					} else {
-						/* extract_short_name returns unescaped */
+						if (file_info->name != NULL) {
+							/* Don't leak if a (potentially malicious)
+							 * server returns several href in its answer
+							 */
+							g_free (file_info->name);
+						}
 						file_info->name = gnome_vfs_uri_extract_short_name (uri);
-						gnome_vfs_uri_unref (uri);
-						
-						len = strlen (file_info->name) -1;
-						if (file_info->name[len] == '/') {
-							/* trim trailing `/` - it confuses stuff */
-							file_info->name[len] = '\0';
+						if (file_info->name != NULL) {
+							len = strlen (file_info->name) -1;
+							if (file_info->name[len] == '/') {
+								/* trim trailing `/` - it confuses stuff */
+								file_info->name[len] = '\0';
+							}
+						} else {
+							g_warning ("Invalid filename in PROPFIND '%s'; silently skipping", nodecontent);
 						}
 					}
+					gnome_vfs_uri_unref (uri);
 				} else {
 					g_warning ("Can't make URI from href in PROPFIND '%s'; silently skipping", nodecontent);
 				}
@@ -2199,6 +2206,7 @@ do_open_directory(GnomeVFSMethod *method,
 
 	if (file_info_cached) {
 		handle = http_file_handle_new (NULL, uri);
+		gnome_vfs_file_info_unref (handle->file_info);
 		handle->file_info = file_info_cached;
 		handle->files = child_file_info_cached_list;
 		result = GNOME_VFS_OK;
