@@ -92,6 +92,40 @@ test_make_canonical_path (const char *input,
 	g_free (output);
 }
 
+const char* test_uris[][2] = 
+{
+	{ "http://www.gnome.org/", "index.html" },
+	{ "http://www.gnome.org/", "/index.html"},
+	{ "http://www.gnome.org/", "/index.html"},
+	{ "http://www.gnome.org", "index.html"},
+	{ "http://www.gnome.org", "/index.html"},
+	{ "http://www.gnome.org", "./index.html"},
+	{NULL, NULL}
+};
+
+
+static void
+test_make_full_from_relative  (const gchar* base, const gchar* relative, 
+			       const gchar* expected_result)
+{
+	GnomeVFSURI *base_uri;
+	GnomeVFSURI *vfs_uri;
+	gchar *str = NULL;
+
+	base_uri = gnome_vfs_uri_new (base);
+	vfs_uri = gnome_vfs_uri_resolve_relative (base_uri, relative);
+	str = gnome_vfs_uri_to_string (vfs_uri, GNOME_VFS_URI_HIDE_NONE);
+	if (expected_result != NULL) {
+		if (strcmp (expected_result, str) != 0) {
+			test_failed ("test_make_full_from_relative (%s, %s) resulted in %s instead of %s\n", base, relative, str, expected_result);
+		}
+	}
+	gnome_vfs_uri_unref (base_uri);
+	gnome_vfs_uri_unref (vfs_uri);
+	g_free (str);
+}
+
+
 static void
 test_uri_to_string (const char *input,
 		    const char *expected_output,
@@ -384,6 +418,8 @@ test_uri_is_parent_shallow (const char *parent, const char *item, gboolean expec
 int
 main (int argc, char **argv)
 {
+	int i;
+
 	make_asserts_break ("GLib");
 	make_asserts_break ("GnomeVFS");
 
@@ -647,8 +683,8 @@ main (int argc, char **argv)
 	test_uri_to_string ("glorp:", "NULL", GNOME_VFS_URI_HIDE_NONE);
 	test_uri_parent ("glorp:", "URI NULL");
 
-	test_uri_to_string ("file:", "file:", GNOME_VFS_URI_HIDE_NONE);
-	test_uri_to_string ("http:", "http:", GNOME_VFS_URI_HIDE_NONE);
+	test_uri_to_string ("file:", "file:///", GNOME_VFS_URI_HIDE_NONE);
+	test_uri_to_string ("http:", "http:///", GNOME_VFS_URI_HIDE_NONE);
 	test_uri_to_string ("file:/", "file:///", GNOME_VFS_URI_HIDE_NONE);
 
 	/* FIXME bugzilla.eazel.com 6022: At least for http, we don't
@@ -672,6 +708,16 @@ main (int argc, char **argv)
 	VERIFY_STRING_RESULT (gnome_vfs_get_local_path_from_uri ("/my/document.html"), NULL);
 	VERIFY_STRING_RESULT (gnome_vfs_get_local_path_from_uri ("http://my/document.html"), NULL);
 	
+	/* Testing gnome_vfs_uri_make_full_from_relative */
+	/* (not an extensive testing, but a regression test */
+	i = 0;
+	while (test_uris[i][0] != NULL) {
+		test_make_full_from_relative (test_uris[i][0], test_uris[i][1],
+					      "http://www.gnome.org/index.html");
+		i++;
+	}
+
+
 	/* Report to "make check" on whether it all worked or not. */
 	return at_least_one_test_failed ? EXIT_FAILURE : EXIT_SUCCESS;
 }
